@@ -71,9 +71,6 @@ class TestView(QQuickView):
         for k, v in self.ctx_prop[0].items():
             self.rootContext().setContextProperty(k, v)
 
-    # def keyPressEvent(self, e):
-    #     breakpoint()
-
 
 class QMLFile(pytest.File):
     def __init__(self, fspath, parent):
@@ -104,7 +101,7 @@ class QMLFile(pytest.File):
 
 class QMLItem(pytest.Item):
     def __init__(self, name, parent, testcase):
-        super().__init__(testcase.name + "::" + name, parent)
+        super().__init__(testcase.name + ":" + name, parent)
         self.testname = name
         self.testcase = testcase
         self.add_marker("qmltest")
@@ -125,7 +122,10 @@ class QMLItem(pytest.Item):
             self.testcase.setProperty("testToRun", self.testname)
 
         # Process result
-        view.close()
+        for win in view.app.allWindows():
+            win.close()
+        view.qmlbot.wait(50)  # wait a little between tests
+
         res = self.testcase.property("result").toVariant()
         self._handle_result(res)
 
@@ -136,7 +136,7 @@ class QMLItem(pytest.Item):
         return (
             self.fspath,
             None,
-            f"{self.parent.name}::{self.name}",
+            f"{self.name}",
         )
 
     def _handle_result(self, result: dict):
