@@ -1,4 +1,5 @@
-from typing import Any, Optional
+import warnings
+from typing import Any, Optional, Union
 
 from pytestqml.qt import (
     QObject,
@@ -13,6 +14,11 @@ from pytestqml.qt import (
     QUrl,
     QVector3D,
     QDateTime,
+    QColor,
+    QGuiApplication,
+    QWheelEvent,
+    QPointF,
+    QWindow,
 )
 from pytestqt.qtbot import QtBot
 
@@ -85,6 +91,11 @@ class QmlBot(QObject):
             return int(lhs == rhs)
         return 2
 
+    @Property(int)
+    def dragThreshold(self):
+        app = QGuiApplication.instance()
+        return app.styleHints().startDragDistance()
+
     @Slot(str, result=bool)
     def isExpectedToFail(self, tag: str):
         return tag in self.expectedToFail
@@ -99,17 +110,88 @@ class QmlBot(QObject):
             return
         return parent.findChild(QObject, objectName)
 
-    @Slot(str, QPoint, int, int, int)
+    @Slot("QVariant", "QVariant", float, result=bool)
+    def fuzzyCompare(
+        self,
+        actual: Union[float, QColor],
+        expected: Union[float, QColor],
+        delta: float = 0.0,
+    ) -> bool:
+        """
+        Compare actual and expected with delta accepted.
+        """
+        if isinstance(actual, bool) or isinstance(expected, bool):
+            return False
+
+        if isinstance(actual, float) and isinstance(expected, float):
+            return abs(actual - expected) <= delta
+
+        try:
+            actual = QColor(actual)
+            expected = QColor(expected)
+        except TypeError:
+            return False
+        if actual.isValid() and expected.isValid():
+            return (
+                abs(actual.green() - expected.green()) <= delta
+                and abs(actual.red() - expected.red()) <= delta
+                and abs(actual.blue() - expected.blue()) <= delta
+                and abs(actual.blue() - expected.blue()) <= delta
+                and abs(actual.alpha() - expected.alpha()) <= delta
+            )
+        return False
+
+    @Slot(str, QPoint, int, int, int, QObject)
     def mouseEvent(
-        self, action: str, point: QPoint, button: int, modifiers: int, delay: int
+        self,
+        action: str,
+        point: QPoint,
+        button: int,
+        modifiers: int,
+        delay: int,
+        rootitem: QObject,
     ):
+        window = self._get_window(rootitem)
+        if not window:
+            raise ValueError("no window found for this element")
         if action == "mouseMove":
-            QTest.mouseMove(self.view, point, delay)
+            QTest.mouseMove(window, point, delay)
         else:
             # Conversion needed for pyqt5/pyside2 compat
             modifiers = Qt.KeyboardModifier(modifiers)
             button = Qt.MouseButton(button)
-            getattr(QTest, action)(self.view, button, modifiers, point, delay)
+            getattr(QTest, action)(window, button, modifiers, point, delay)
+
+    @Slot(QPointF, QPointF, QPoint, int, int, int, QObject)
+    def mouseWheel(
+        self, pos, globalPos, angleDelta, buttons, modifiers, delay, rootitem: QObject
+    ):
+
+        window = self._get_window(rootitem)
+        if not window:
+            raise ValueError("no window found for this element")
+        modifiers = Qt.KeyboardModifier(modifiers)
+        buttons = Qt.MouseButton(buttons)
+        pixelDelta = QPoint()
+        phase = Qt.NoScrollPhase
+        inverted = False
+        source = Qt.MouseEventNotSynthesized
+
+        event = QWheelEvent(
+            pos,
+            globalPos,
+            pixelDelta,
+            angleDelta,
+            buttons,
+            modifiers,
+            phase,
+            inverted,
+            source,
+        )
+        if delay > 0:
+            self.wait(delay)
+        QGuiApplication.instance().postEvent(window, event)
+        QGuiApplication.instance().processEvents()
 
     @Slot(str, int, int, int)
     def keyEvent(
@@ -123,11 +205,13 @@ class QmlBot(QObject):
         # print(modifiers)
         key = Qt.Key(key)
         modifiers = Qt.KeyboardModifiers(modifiers)
-        getattr(QTest, action)(self.view, key, modifiers, delay)
+        window = self.view.app.focusWindow() or self.view
+        getattr(QTest, action)(window, key, modifiers, delay)
 
     @Slot(str)
     def keySequence(self, seq: str):
-        QTest.keySequence(self.view, QKeySequence(seq))
+        window = self.view.app.focusWindow() or self.view
+        QTest.keySequence(window, QKeySequence(seq))
 
     windowShownChanged = Signal()
 
@@ -152,6 +236,10 @@ class QmlBot(QObject):
             value = value.toVariant()
         self._settings[key] = value
 
+    @Slot(str)
+    def warn(self, value: str):
+        warnings.warn_explicit(value, QMLWarning, "", 0)
+
     #
     # @Slot("QVariant", result=str)
     # def stringify(self, value: Any):
@@ -178,36 +266,6 @@ class QmlBot(QObject):
     #             result += str(value)
     #     return result
     #
-    # @Slot(float, float, float, result=bool)
-    # @Slot(float, float, result=bool)
-    # @Slot(QColor, QColor, result=bool)
-    # @Slot(QColor, str, result=bool)
-    # @Slot(str, QColor, result=bool)
-    # @Slot(QColor, str, float, result=bool)
-    # @Slot(str, QColor, float, result=bool)
-    # @Slot(QColor, QColor, float, result=bool)
-    # @Slot(str, str, float, result=bool)
-    # def fuzzyCompare(
-    #     self, actual: fuzzyType, expected: fuzzyType, delta: float = 0.0
-    # ) -> bool:
-    #     """
-    #     Compare actual and expected with delta accepted.
-    #     """
-    #     if isinstance(actual, float) and isinstance(expected, float):
-    #         return abs(actual - expected) <= delta
-    #     else:
-    #         act = QColor(actual)
-    #         exp = QColor(expected)
-    #
-    #         if not act.isValid() or not exp.isValid():
-    #             return False
-    #         return (
-    #             abs(act.green() - exp.green()) <= delta
-    #             and abs(act.red() - exp.red()) <= delta
-    #             and abs(act.blue() - exp.blue()) <= delta
-    #             and abs(act.blue() - exp.blue()) <= delta
-    #             and abs(act.alpha() - exp.alpha()) <= delta
-    #         )
 
     """
     Private api
@@ -217,3 +275,11 @@ class QmlBot(QObject):
     #     window = self.engine.rootObjects()[0]
     #     QTest.mousePress(window, Qt.LeftButton, Qt.NoModifier, QPoint(20, 10), 0)
     # def _track_key(self, key):
+    def _get_window(self, rootitem: QObject) -> QWindow:
+        for win in self.view.app.allWindows():
+            if win.contentItem() == rootitem:
+                return win
+
+
+class QMLWarning(UserWarning):
+    pass
