@@ -42,6 +42,8 @@ QColor = QtGui.QColor
 QGuiApplication = QtGui.QGuiApplication
 QKeySequence = QtGui.QKeySequence
 QVector3D = QtGui.QVector3D
+QWheelEvent = QtGui.QWheelEvent
+QWindow = QtGui.QWindow
 
 # QtQuick
 QQuickView = QtQuick.QQuickView
